@@ -1,84 +1,27 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2010 Carsten Gnoerlich.
- *  Project home page: http://www.dvdisaster.com
- *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
+ *  Copyright (C) 2004-2015 Carsten Gnoerlich.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  Email: carsten@dvdisaster.org  -or-  cgnoerlich@fsfe.org
+ *  Project homepage: http://www.dvdisaster.org
+ *
+ *  This file is part of dvdisaster.
+ *
+ *  dvdisaster is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  dvdisaster is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA,
- *  or direct your browser at http://www.gnu.org.
+ *  along with dvdisaster. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "dvdisaster.h"
 
-#ifdef SYS_MINGW
- #include <windows.h>
- #include <winnls.h>
-#endif
-
-/*
- * Create the error correction file
- */
-
-void CreateEcc(void)
-{  Method *method = FindMethod(Closure->methodName); 
-
-   /*** GUI mode does its own FindMethod() before calling us
-	so the following Stop() will never execute in GUI mode */
-
-   if(!method) Stop(_("\nMethod %s not available.\n"
-		      "Use -m without parameters for a method list.\n"), 
-	            Closure->methodName);
-
-   method->create(method);
-}
-
-/*
- * Fix the medium with ecc information
- */
-
-void FixEcc(void)
-{  Method *method; 
-   
-   /* Error handling is done within EccMethod() */
-
-   method = EccMethod(TRUE);
- 
-   /* Dispatch to the proper method */
-  
-   method->fix(method);
-}
-
-/*
- * Verifiy the image against ecc data 
- */
-
-void Verify(void)
-{  Method *method; 
-   
-  /* If something is wrong with the .iso or .ecc files
-     we fall back to the RS01 method for comparing 
-     since it is robust against missing files. */
-
-  if(!(method = EccMethod(FALSE)))
-    if(!(method = FindMethod("RS01")))
-      Stop(_("RS01 method not available for comparing files."));
- 
-   /* Dispatch to the proper method */
-
-  method->verify(method);
-}
-
 /*
  * The all-famous main() loop 
  */
@@ -100,14 +43,15 @@ typedef enum
    MODE_DEBUG_MAINT1,
    MODE_ERASE, 
    MODE_MARKED_IMAGE,
+   MODE_MEDIUM_INFO,
    MODE_MERGE_IMAGES,
    MODE_RANDOM_ERR, 
    MODE_RANDOM_IMAGE,
    MODE_RAW_SECTOR,
    MODE_READ_SECTOR,
    MODE_SEND_CDB,
+   MODE_SHOW_HEADER,
    MODE_SHOW_SECTOR, 
-   MODE_SIGN,
    MODE_TRUNCATE,
    MODE_ZERO_UNREADABLE,
 
@@ -122,17 +66,24 @@ typedef enum
    MODIFIER_DEFECTIVE_DUMP,
    MODIFIER_DRIVER,
    MODIFIER_EJECT,
+   MODIFIER_ENCODING_ALGORITHM,
+   MODIFIER_ENCODING_IO_STRATEGY,
    MODIFIER_FILL_UNREADABLE,
+   MODIFIER_FIXED_SPEED_VALUES,
    MODIFIER_IGNORE_FATAL_SENSE,
+   MODIFIER_IGNORE_ISO_SIZE,
    MODIFIER_INTERNAL_REREADS,
    MODIFIER_OLD_DS_MARKER,
    MODIFIER_PREFETCH_SECTORS,
    MODIFIER_RANDOM_SEED,
+   MODIFIER_RAW_MODE,
    MODIFIER_READ_ATTEMPTS,
    MODIFIER_READ_MEDIUM,
    MODIFIER_READ_RAW,
-   MODIFIER_RAW_MODE,
+   MODIFIER_RESOURCE_FILE,
    MODIFIER_SCREEN_SHOT,
+   MODIFIER_SET_VERSION,
+   MODIFIER_SIMULATE_CD,
    MODIFIER_SIMULATE_DEFECTS,
    MODIFIER_SPEED_WARNING, 
    MODIFIER_SPINUP_DELAY, 
@@ -149,32 +100,15 @@ int main(int argc, char *argv[])
 #ifdef WITH_NLS_YES
    char *locale_test;
  #ifdef WITH_EMBEDDED_SRC_PATH_YES
-  #ifndef SYS_MINGW
    char src_locale_path[strlen(SRCDIR)+10];
-  #endif
  #endif /* WITH_EMBEDDED_SRC_PATH_YES */
-#if defined(SYS_MINGW) || defined(SYS_DARWIN)
-   char *bin_locale_path = NULL;
- #endif
 #endif
+   int debug_mode_required=FALSE;
 
 #ifdef WITH_MEMDEBUG_YES
     atexit(check_memleaks);
 #endif
 
-    /*** Do this early so that we can use mutexes */
-
-    g_thread_init(NULL);
-
-#ifdef SYS_MINGW
-    /*** Create a named mutex so that the installer can detect
-	 that we are running. A malicious user may cause this 
-	 to fail by reserving the mutex for himself, 
-	 so we do not care if the mutex can not be created. */
-
-    CreateMutex(NULL, FALSE, "dvdisaster");
-#endif
-
     /*** Setup the global closure. */
 
     InitClosure();
@@ -191,110 +125,11 @@ int main(int argc, char *argv[])
 
 #ifdef WITH_NLS_YES
 
-#if 0 //WIN_CONSOLE
-    /* We need to manually set the code page when running
-       in the console */
-
-    if(!g_getenv("OUTPUT_CHARSET"))  /* User may override this */
-    {  gchar *unix_locale = g_win32_getlocale();
-
-      if(!strncmp(unix_locale, "cs", 2))
-	g_setenv("OUTPUT_CHARSET", "CP852", 1);
-
-      if(!strncmp(unix_locale, "de", 2))
-	g_setenv("OUTPUT_CHARSET", "CP850", 1);
-
-      if(!strncmp(unix_locale, "it", 2))
-	g_setenv("OUTPUT_CHARSET", "CP850", 1);
-
-      if(!strncmp(unix_locale, "pt_BR", 4))
-	g_setenv("OUTPUT_CHARSET", "CP850", 1);
-
-      if(!strncmp(unix_locale, "ru", 2))
-	g_setenv("OUTPUT_CHARSET", "CP855", 1);
-
-      if(!strncmp(unix_locale, "sv", 2))
-	g_setenv("OUTPUT_CHARSET", "CP850", 1);
-    }
-#endif /* WIN_CONSOLE */
-
-#ifdef SYS_MINGW
-    if(!g_getenv("LANG"))  /* Unix style setting has precedence */
-    {  LANGID lang_id;
-
-       /* Try to get locale from Windows
-	  and set the respective environment variables. */
-
-       lang_id = GetUserDefaultLangID(); 
-       PrintLog("GetUserDefaultLangID = %x; Primary: %d, Sub: %d\n",
-		lang_id, PRIMARYLANGID(lang_id), SUBLANGID(lang_id));
-
-       switch(PRIMARYLANGID(lang_id))
-       {  case LANG_CZECH:
-	     g_setenv("LANG", "cs_CZ", 1);
-#ifdef WIN_CONSOLE
-	     g_setenv("OUTPUT_CHARSET", "CP852", 1);
-#else
-	     g_setenv("OUTPUT_CHARSET", "CP1250", 1);
-#endif
-	     break;
-
-          case LANG_GERMAN:
-	     g_setenv("LANG", "de_DE", 1);
-#ifdef WIN_CONSOLE
-	     g_setenv("OUTPUT_CHARSET", "CP850", 1);
-#else
-	     g_setenv("OUTPUT_CHARSET", "CP1252", 1);
-#endif
-	     break;
-
-          case LANG_ITALIAN:
-	     g_setenv("LANG", "it_IT", 1);
-#ifdef WIN_CONSOLE
-	     g_setenv("OUTPUT_CHARSET", "CP850", 1);
-#else
-	     g_setenv("OUTPUT_CHARSET", "CP1252", 1);
-#endif
-	     break;
-
-          case LANG_PORTUGUESE:
-	    if(SUBLANGID(lang_id) == SUBLANG_PORTUGUESE_BRAZILIAN)
-	    {  g_setenv("LANG", "pt_BR", 1);
-#ifdef WIN_CONSOLE
-	       g_setenv("OUTPUT_CHARSET", "CP860", 1);
-#else
-	       g_setenv("OUTPUT_CHARSET", "CP1252", 1);
-#endif
-	    }
-            break;
-
-          case LANG_RUSSIAN:
-	     g_setenv("LANG", "ru_RU", 1);
-#ifdef WIN_CONSOLE
-	     g_setenv("OUTPUT_CHARSET", "CP855", 1);
-#else
-	     g_setenv("OUTPUT_CHARSET", "CP1251", 1);
-#endif
-	     break;
-
-          case LANG_SWEDISH:
-	     g_setenv("LANG", "sv_SV", 1);
-#ifdef WIN_CONSOLE
-	     g_setenv("OUTPUT_CHARSET", "CP850", 1);
-#else
-	     g_setenv("OUTPUT_CHARSET", "CP1252", 1);
-#endif
-	     break;
-       }
-    }
-#endif /* SYS_MINGW */
-
     /* This is necessary, but feels broken */
     setlocale(LC_CTYPE, "");
     setlocale(LC_MESSAGES, "");
     textdomain("dvdisaster");
 
-#ifndef SYS_MINGW
     /* Try local source directory first */
 
 #ifdef WITH_EMBEDDED_SRC_PATH_YES
@@ -320,22 +155,6 @@ int main(int argc, char *argv[])
 	  locale_test = gettext("test phrase for verifying the locale installation");
        }
     }
-#endif
-
-#if defined(SYS_MINGW) || defined(SYS_DARWIN)
-    /* Try the directory where our executable comes from.
-       This is only possible under Windows and Mac OS, 
-       and should cover all cases. */
-
-#ifdef SYS_MINGW
-    bin_locale_path = g_strdup_printf("%s\\locale", Closure->binDir);
-#else
-    bin_locale_path = g_strdup_printf("%s/locale", Closure->binDir);
-#endif
-    bindtextdomain("dvdisaster", bin_locale_path);
-    locale_test = gettext("test phrase for verifying the locale installation");
-    g_free(bin_locale_path);
-#endif
 
     /* Last resort: fall back to global locale */
 
@@ -357,6 +176,18 @@ int main(int argc, char *argv[])
    if(sizeof(EccHeader) != 4096)
      Stop("sizeof(EccHeader) is %d, but must be 4096.\n", sizeof(EccHeader));
 
+   /*** If we have too much command line options fail here */
+
+   if(MODIFIER_VERSION >= 'a')
+     Stop("Too many command line options\n");
+
+   /*** CPU type detection. Must be done before parsing the options
+        as some may be CPU-related. */
+
+   Closure->useSSE2 = ProbeSSE2();
+   Closure->useAltiVec = ProbeAltiVec();
+   Closure->clSize = ProbeCacheLineSize();
+
    /*** Parse the options */
    
    for(;;)
@@ -382,18 +213,20 @@ int main(int argc, char *argv[])
         {"ecc", 1, 0, 'e'},
 	{"ecc-target", 1, 0, 'o'},
 	{"eject", 0, 0, MODIFIER_EJECT },
+	{"encoding-algorithm", 1, 0, MODIFIER_ENCODING_ALGORITHM },
+	{"encoding-io-strategy", 1, 0, MODIFIER_ENCODING_IO_STRATEGY },
 	{"erase", 1, 0, MODE_ERASE },
 	{"fill-unreadable", 1, 0, MODIFIER_FILL_UNREADABLE },
 	{"fix", 0, 0, 'f'},
+	{"fixed-speed-values", 0, 0, MODIFIER_FIXED_SPEED_VALUES },
 	{"help", 0, 0, 'h'},
 	{"ignore-fatal-sense", 0, 0, MODIFIER_IGNORE_FATAL_SENSE },
+	{"ignore-iso-size", 0, 0, MODIFIER_IGNORE_ISO_SIZE },
 	{"internal-rereads", 1, 0, MODIFIER_INTERNAL_REREADS },
         {"image", 1, 0, 'i'},
 	{"jump", 1, 0, 'j'},
-#ifdef SYS_MINGW
-	{"list", 0, 0, 'l' },
-#endif
 	{"marked-image", 1, 0, MODE_MARKED_IMAGE },
+	{"medium-info", 0, 0, MODE_MEDIUM_INFO },
 	{"merge-images", 1, 0, MODE_MERGE_IMAGES },
 	{"method", 2, 0, 'm' },
 	{"old-ds-marker", 0, 0, MODIFIER_OLD_DS_MARKER },
@@ -410,11 +243,14 @@ int main(int argc, char *argv[])
 	{"read-sector", 1, 0, MODE_READ_SECTOR},
 	{"read-raw", 0, 0, MODIFIER_READ_RAW},
 	{"redundancy", 1, 0, 'n'},
+	{"resource-file", 1, 0, MODIFIER_RESOURCE_FILE},
 	{"scan", 2, 0,'s'},
 	{"screen-shot", 0, 0, MODIFIER_SCREEN_SHOT },
+	{"set-version", 1, 0, MODIFIER_SET_VERSION},
 	{"send-cdb", 1, 0, MODE_SEND_CDB},
+	{"show-header", 1, 0, MODE_SHOW_HEADER},
 	{"show-sector", 1, 0, MODE_SHOW_SECTOR},
-	{"sign", 0, 0, MODE_SIGN},
+	{"sim-cd", 2, 0, MODIFIER_SIMULATE_CD},
 	{"sim-defects", 1, 0, MODIFIER_SIMULATE_DEFECTS},
 	{"speed-warning", 2, 0, MODIFIER_SPEED_WARNING},
 	{"spinup-delay", 1, 0, MODIFIER_SPINUP_DELAY},
@@ -435,7 +271,7 @@ int main(int argc, char *argv[])
       if(c == -1) break;
 
       switch(c)
-      {            /* Mingw has not strcasestr() */
+      {            
 	 case 'a': if(strstr(optarg, "rs02") || strstr(optarg, "RS02"))
 		      Closure->examineRS02 = TRUE; 
 	           if(strstr(optarg, "rs03") || strstr(optarg, "RS03"))
@@ -443,12 +279,14 @@ int main(int argc, char *argv[])
 		   break;	       
          case 'c': mode = MODE_SEQUENCE; sequence |= 1<<MODE_CREATE; break;
          case 'd': if(optarg) 
-	           {  g_free(Closure->device);
+		   {  if(Closure->device)
+			 g_free(Closure->device);
 	              Closure->device = g_strdup(optarg); 
 	              break;
                    }
          case 'e': if(optarg) 
-	           {  g_free(Closure->eccName);
+		   {  if(Closure->eccName)
+		         g_free(Closure->eccName);
 		      Closure->eccName = g_strdup(optarg);
 		   }
 	           break;
@@ -529,14 +367,15 @@ int main(int argc, char *argv[])
 	   Closure->autoSuffix = TRUE;
 	   break;
          case MODIFIER_CACHE_SIZE:
-	   Closure->cacheMB = atoi(optarg);
-	   if(Closure->cacheMB <   8) 
-	     Stop(_("--cache-size must at least be 8MB; 16MB or higher is recommended.")); 
-	   if(Closure->cacheMB > 8192) 
-	     Stop(_("--cache-size maximum is 8192MB.")); 
+	   Closure->cacheMiB = atoi(optarg);
+	   if(Closure->cacheMiB <   8) 
+	     Stop(_("--cache-size must at least be 8MiB; 16MiB or higher is recommended.")); 
+	   if(Closure->cacheMiB > MAX_OLD_CACHE_SIZE) 
+	      Stop(_("--cache-size maximum is %dMiB."), MAX_OLD_CACHE_SIZE); 
 	   break;
          case MODIFIER_CDUMP:
 	   Closure->debugCDump = TRUE;
+	   debug_mode_required = TRUE;
 	   break;
          case MODIFIER_DAO: 
 	   Closure->noTruncate = 1; 
@@ -544,6 +383,50 @@ int main(int argc, char *argv[])
          case MODIFIER_EJECT: 
 	   Closure->eject = 1; 
 	   break;
+         case MODIFIER_ENCODING_ALGORITHM:
+	   Closure->encodingAlgorithm = ENCODING_ALG_INVALID;
+	   if(!strcmp(optarg, "32bit"))
+	     Closure->encodingAlgorithm = ENCODING_ALG_32BIT;
+	   if(!strcmp(optarg, "64bit"))
+	     Closure->encodingAlgorithm = ENCODING_ALG_64BIT;
+#ifdef HAVE_SSE2
+	   if(!strcmp(optarg, "SSE2"))
+	   {  Closure->encodingAlgorithm = ENCODING_ALG_SSE2;
+
+	     if(!Closure->useSSE2)
+	       Stop(_("--encoding-algorithm: SSE2 not supported on this processor!"));
+	   }
+
+	   if(Closure->encodingAlgorithm == ENCODING_ALG_INVALID)
+	     Stop(_("--encoding-algorithm: valid types are 32bit, 64bit, SSE2"));
+#endif
+#ifdef HAVE_ALTIVEC
+	   if(!strcmp(optarg, "AltiVec"))
+	   {  Closure->encodingAlgorithm = ENCODING_ALG_ALTIVEC;
+
+	     if(!Closure->useAltiVec)
+	       Stop(_("--encoding-algorithm: AltiVec not supported on this processor!"));
+	   }
+
+	   if(Closure->encodingAlgorithm == ENCODING_ALG_INVALID)
+	     Stop(_("--encoding-algorithm: valid types are 32bit, 64bit, AltiVec"));
+#endif
+	   if(Closure->encodingAlgorithm == ENCODING_ALG_INVALID)
+	     Stop(_("--encoding-algorithm: valid types are 32bit, 64bit"));
+	   break;
+	 case MODIFIER_ENCODING_IO_STRATEGY:
+	   if(!strcmp(optarg, "readwrite"))
+	   {  Closure->encodingIOStrategy = IO_STRATEGY_READWRITE;
+	   }
+	   else if(!strcmp(optarg, "mmap"))
+	   {  Closure->encodingIOStrategy = IO_STRATEGY_MMAP;
+#ifndef HAVE_MMAP
+	      Stop(_("--encoding-io-strategy: mmap not supported on this OS"));
+#endif
+	   }
+	   else
+	      Stop(_("--encoding-io-strategy: valid types are readwrite and mmap"));
+	   break;
 	 case MODIFIER_DRIVER:
 #if defined(SYS_LINUX)
 	   if(optarg && !strcmp(optarg,"sg"))
@@ -560,9 +443,18 @@ int main(int argc, char *argv[])
          case MODIFIER_FILL_UNREADABLE:
 	   if(optarg) Closure->fillUnreadable = strtol(optarg, NULL, 0);
 	   break;
+         case MODIFIER_FIXED_SPEED_VALUES:
+	    if(!Closure->debugMode)
+	      Stop(_("--fixed-speed-values is only allowed in debug mode"));
+	    Closure->fixedSpeedValues=TRUE;
+ 	    debug_mode_required = TRUE;
+	    break;
          case MODIFIER_IGNORE_FATAL_SENSE:
 	   Closure->ignoreFatalSense = TRUE;
 	   break;
+         case MODIFIER_IGNORE_ISO_SIZE:
+	   Closure->ignoreIsoSize = TRUE;
+	   break;
 	 case MODIFIER_INTERNAL_REREADS:
 	    if(optarg)
 	       Closure->internalAttempts = atoi(optarg);
@@ -571,13 +463,6 @@ int main(int argc, char *argv[])
 	    if(Closure->internalAttempts > 10) 
 	       Closure->internalAttempts = 10;
 	    break;
-         case MODIFIER_TRUNCATE: 
-	   if(optarg)                  /* debugging truncate mode */
-	   {  mode = MODE_TRUNCATE;
-	      debug_arg = g_strdup(optarg);
-	   }
-	   else Closure->truncate = 1; /* truncate confirmation for fix mode */
-	   break;
          case MODIFIER_DEBUG:
 	   Closure->debugMode = TRUE;
 	   break;
@@ -600,12 +485,13 @@ int main(int argc, char *argv[])
          case MODIFIER_PREFETCH_SECTORS:
  	    Closure->prefetchSectors = atoi(optarg);
 	    if(   Closure->prefetchSectors < 32
-	       || Closure->prefetchSectors > 8096)
-	      Stop(_("--prefetch-sectors must be in range 32...8096"));
+	       || Closure->prefetchSectors > MAX_PREFETCH_CACHE_SIZE)
+	      Stop(_("--prefetch-sectors must be in range 32...%s"),
+		    MAX_PREFETCH_CACHE_SIZE);
 	    break;
-
          case MODIFIER_RANDOM_SEED:
 	   if(optarg) Closure->randomSeed = atoi(optarg);
+	   debug_mode_required = TRUE;
 	   break;
          case MODIFIER_RAW_MODE:
 	    if(optarg) Closure->rawMode = strtol(optarg,NULL,16);
@@ -639,12 +525,34 @@ int main(int argc, char *argv[])
          case MODIFIER_READ_RAW:
 	   Closure->readRaw = TRUE;
 	   break;
+         case MODIFIER_RESOURCE_FILE:
+	   if(Closure->dotFile)
+	     g_free(Closure->dotFile);
+	   Closure->dotFile = g_strdup(optarg);
+	   break;
          case MODIFIER_SCREEN_SHOT:
 	   Closure->screenShotMode = TRUE;
+	   debug_mode_required = TRUE;
 	   break;
-         case MODIFIER_SIMULATE_DEFECTS:
+         case MODIFIER_SET_VERSION:
+	 {  int v1,v2;
+	    if(!Closure->debugMode)
+	      Stop(_("--set-version is only allowed in debug mode"));
+	    g_free(Closure->cookedVersion);
+	    Closure->cookedVersion = g_strdup(optarg);
+	    sscanf(Closure->cookedVersion,"%d.%d",&v1,&v2);
+	    Closure->version=10000*v1+100*v2;
+ 	    debug_mode_required = TRUE;
+	 } 
+	    break;
+        case MODIFIER_SIMULATE_CD:
+	   if(optarg) Closure->simulateCD = g_strdup(optarg);
+	   debug_mode_required = TRUE;
+	   break;
+        case MODIFIER_SIMULATE_DEFECTS:
 	   if(optarg) Closure->simulateDefects = atoi(optarg);
 	   else Closure->simulateDefects = 10;
+	   debug_mode_required = TRUE;
 	   break;
          case MODIFIER_SPINUP_DELAY:
 	   if(optarg) Closure->spinupDelay = atoi(optarg);
@@ -653,6 +561,14 @@ int main(int argc, char *argv[])
 	   if(optarg) Closure->speedWarning = atoi(optarg);
 	   else Closure->speedWarning=10;
 	   break;
+         case MODIFIER_TRUNCATE: 
+	   if(optarg)                  /* debugging truncate mode */
+	   {  mode = MODE_TRUNCATE;
+	      debug_arg = g_strdup(optarg);
+	      debug_mode_required = TRUE;
+	   }
+	   else Closure->truncate = 1; /* truncate confirmation for fix mode */
+	   break;
          case MODIFIER_CLV_SPEED:
 	   Closure->driveSpeed = atoi(optarg);
 	   break;
@@ -660,11 +576,10 @@ int main(int argc, char *argv[])
 	   Closure->driveSpeed = -atoi(optarg);
 	   break;
          case MODIFIER_VERSION:
-	   PrintCLI(_("\ndvdisaster version %s build %d\n\n"), 
-		    Closure->cookedVersion, buildCount);
-	   FreeClosure();
-	   exit(EXIT_SUCCESS); 
-	   break;
+	    PrintCLI("\n%s\n\n", Closure->versionString);
+	    FreeClosure();
+	    exit(EXIT_SUCCESS); 
+	    break;
          case MODE_BYTESET:
 	   mode = MODE_BYTESET;
 	   debug_arg = g_strdup(optarg);
@@ -689,6 +604,10 @@ int main(int argc, char *argv[])
 	   mode = MODE_MARKED_IMAGE;
 	   debug_arg = g_strdup(optarg);
 	   break;
+         case MODE_MEDIUM_INFO:
+	   mode = MODE_MEDIUM_INFO;
+	   debug_arg = g_strdup(optarg);
+	   break;
          case MODE_MERGE_IMAGES:
 	   mode = MODE_MERGE_IMAGES;
 	   debug_arg = g_strdup(optarg);
@@ -713,8 +632,9 @@ int main(int argc, char *argv[])
 	   mode = MODE_SEND_CDB;
 	   debug_arg = g_strdup(optarg);
 	   break;
-         case MODE_SIGN:
-	   mode = MODE_SIGN;
+         case MODE_SHOW_HEADER:
+	   mode = MODE_SHOW_HEADER;
+	   debug_arg = g_strdup(optarg);
 	   break;
          case MODE_SHOW_SECTOR:
 	   mode = MODE_SHOW_SECTOR;
@@ -731,6 +651,9 @@ int main(int argc, char *argv[])
    /*** Don't allow debugging option if --debug wasn't given */
    
    if(!Closure->debugMode)
+   { if(debug_mode_required)
+	 mode=MODE_HELP;
+
      switch(mode)
      {  case MODE_BYTESET:
 	case MODE_COPY_SECTOR:
@@ -741,33 +664,17 @@ int main(int argc, char *argv[])
         case MODE_READ_SECTOR:
         case MODE_RAW_SECTOR:
         case MODE_SEND_CDB:
+        case MODE_SHOW_HEADER:
         case MODE_SHOW_SECTOR:
         case MODE_MARKED_IMAGE:
         case MODE_MERGE_IMAGES:
-        case MODE_SIGN:
         case MODE_TRUNCATE:
         case MODE_ZERO_UNREADABLE:
 	  mode = MODE_HELP;
           break;
      }
-	  
-#ifdef WIN_CONSOLE
-   if(mode != MODE_SIGN && !VerifySignature())
-   {  char version[80];
-
-      if(Closure->version % 100)
-           sprintf(version, "dvdisaster-%s.%d-setup.exe", VERSION, Closure->version%100);
-      else sprintf(version, "dvdisaster-%s-setup.exe", VERSION);
-      Stop(_("dvdisaster is not properly installed.\n"
-	     "Please execute the installer program (%s) again.\n"), version);
    }
-#endif
-
-   /*** CPU type detection. */
-
-   Closure->useSSE2 = ProbeSSE2();
-   Closure->useAltiVec = ProbeAltiVec();
-
+	  
    /*** Parse the sector ranges for --read and --scan */
 
    if(read_range)
@@ -810,15 +717,7 @@ int main(int argc, char *argv[])
 
    /*** Dispatch action depending on mode.
         The major modes can be executed in sequence, 
-	but not all combinations may be really useful.
-
-	The GUI version for Windows does not have an open console,
-	so no command line actions can be carried out. 
-	Force opening the GUI by clearing the mode variable. */
-
-#if defined(SYS_MINGW) && !defined(WIN_CONSOLE)
-   mode = MODE_NONE;
-#endif
+	but not all combinations may be really useful. */
 
    switch(mode)
    {  case MODE_SEQUENCE:
@@ -834,13 +733,58 @@ int main(int argc, char *argv[])
 	}
 
 	if(sequence & 1<<MODE_CREATE)
-	  CreateEcc();
+	{  Method *method = FindMethod(Closure->methodName); 
+
+	   if(!method) Stop(_("\nMethod %s not available.\n"
+			      "Use -m without parameters for a method list.\n"), 
+			    Closure->methodName);
+
+	   method->create();
+	}
 
 	if(sequence & 1<<MODE_FIX)
-	  FixEcc();
+	{  Method *method = NULL;
+	   Image *image;
+
+	   PrintLog(_("\nOpening %s"), Closure->imageName);
+	   image = OpenImageFromFile(Closure->imageName, O_RDWR, IMG_PERMS);
+	   if(!image)
+	   {  PrintLog(": %s.\n", strerror(errno));
+	   }
+	   else 
+	   {  if(image->inLast == 2048)
+	           PrintLog(_(": %lld medium sectors.\n"), image->sectorSize);
+	      else PrintLog(_(": %lld medium sectors and %d bytes.\n"), 
+		   image->sectorSize-1, image->inLast);
+	   }
+	   image = OpenEccFileForImage(image, Closure->eccName, O_RDWR, IMG_PERMS);
+	   ReportImageEccInconsistencies(image);
+
+	   /* Determine method. Ecc files win over augmented ecc. */
+
+	   if(image && image->eccFileMethod) method = image->eccFileMethod;
+	   else if(image && image->eccMethod) method = image->eccMethod;
+	   else Stop("Internal error: No suitable method for repairing image.");
+
+	   method->fix(image);
+	}
 
 	if(sequence & 1<<MODE_VERIFY)
-	  Verify();
+	{  Method *method;
+	   Image *image;
+
+	   image = OpenImageFromFile(Closure->imageName, O_RDONLY, IMG_PERMS);
+	   image = OpenEccFileForImage(image, Closure->eccName, O_RDONLY, IMG_PERMS);
+
+	   /* Determine method. Ecc files win over augmented ecc. */
+
+	   if(image && image->eccFileMethod) method = image->eccFileMethod;
+	   else if(image && image->eccMethod) method = image->eccMethod;
+	   else if(!(method = FindMethod("RS01")))
+	           Stop(_("RS01 method not available for comparing files."));
+	     
+	   method->verify(image);
+	}
 	break;
 
       case MODE_BYTESET:
@@ -878,18 +822,27 @@ int main(int argc, char *argv[])
 	 ReadSector(debug_arg);
 	 break;
 
+      case MODE_SHOW_HEADER:
+	 ShowHeader(debug_arg);
+	 break;
+
       case MODE_SHOW_SECTOR:
 	 ShowSector(debug_arg);
 	 break;
 
       case MODE_RANDOM_ERR:
-	 RandomError(Closure->imageName, debug_arg);
+	 RandomError(debug_arg);
 	 break;
 
       case MODE_MARKED_IMAGE:
 	 RandomImage(Closure->imageName, debug_arg, 1);
 	 break;
 
+      case MODE_MEDIUM_INFO:
+         if(!Closure->device) Closure->device = DefaultDevice();
+	 PrintMediumInfo(NULL);
+	 break;
+
       case MODE_MERGE_IMAGES:
 	 MergeImages(debug_arg, TRUE);
 	 break;
@@ -899,19 +852,13 @@ int main(int argc, char *argv[])
 	 break;
 
       case MODE_TRUNCATE:
-	 TruncateImage(debug_arg);
+	 TruncateImageFile(debug_arg);
 	 break;
 
       case MODE_ZERO_UNREADABLE:
 	 ZeroUnreadable();
 	 break;
 
-#ifdef SYS_MINGW
-      case MODE_SIGN:
-	 WriteSignature();
-	 exit(0);
-	 break;
-#endif
       default:
 	break;
    }
@@ -920,16 +867,12 @@ int main(int argc, char *argv[])
 
    /*** If no mode was selected, print the help screen. */
 
-#ifdef WIN_CONSOLE
-   if(mode == MODE_HELP || mode == MODE_NONE)
-#else
    if(mode == MODE_HELP)
-#endif
    {  
      /* TRANSLATORS: Program options like -r and --read are not to be translated
 	to avoid confusion when discussing the program in international forums. */
       PrintCLI(_("\nCommon usage examples:\n"
-	     "  dvdisaster -r,--read   # Read the medium image to hard disc.\n"
+	     "  dvdisaster -r,--read   # Read the medium image to hard disk.\n"
 	     "                         # Use -rn-m to read a certain sector range, e.g. -r100-200\n"
 	     "  dvdisaster -c,--create # Create .ecc information for the medium image.\n"
 	     "  dvdisaster -f,--fix    # Try to fix medium image using .ecc information.\n"
@@ -957,22 +900,27 @@ int main(int argc, char *argv[])
       PrintCLI(_("  -x,--threads n         - use n threads for en-/decoding (if supported by codec)\n"));
       PrintCLI(_("  --adaptive-read        - use optimized strategy for reading damaged media\n"));
       PrintCLI(_("  --auto-suffix          - automatically add .iso and .ecc file suffixes\n"));
-      PrintCLI(_("  --cache-size n         - image cache size in MB during -c mode (default: 32MB)\n"));
+      PrintCLI(_("  --cache-size n         - image cache size in MiB during -c mode (default: 32MiB)\n"));
       PrintCLI(_("  --dao                  - assume DAO disc; do not trim image end\n"));
       PrintCLI(_("  --defective-dump d     - directory for saving incomplete raw sectors\n"));
 #ifdef SYS_LINUX
       PrintCLI(_("  --driver=sg/cdrom      - use sg(default) or alternative cdrom driver (see man page!)\n"));
 #endif
       PrintCLI(_("  --eject                - eject medium after successful read\n"));
+      PrintCLI(_("  --encoding-algorithm n - possible values: 32bit,64bit,SSE2,AltiVec\n"));
+      PrintCLI(_("  --encoding-io-strategy n - possible values: readwrite, mmap\n"));
       PrintCLI(_("  --fill-unreadable n    - fill unreadable sectors with byte n\n"));
       PrintCLI(_("  --ignore-fatal-sense   - continue reading after potentially fatal error conditon\n"));
+      PrintCLI(_("  --ignore-iso-size      - ignore image size from ISO/UDF data (dangerous - see man page!)\n"));
       PrintCLI(_("  --internal-rereads n   - drive may attempt n rereads before reporting an error\n"));
+      PrintCLI(_("  --medium-info          - print info about medium in drive\n"));
       PrintCLI(_("  --old-ds-marker        - mark missing sectors compatible with dvdisaster <= 0.70\n"));
-      PrintCLI(_("  --prefetch-sectors n   - prefetch n sectors for RS03 encoding (uses ~nMB)\n"));
+      PrintCLI(_("  --prefetch-sectors n   - prefetch n sectors for RS03 encoding (uses ~nMiB)\n"));
       PrintCLI(_("  --raw-mode n           - mode for raw reading CD media (20 or 21)\n"));
       PrintCLI(_("  --read-attempts n-m    - attempts n upto m reads of a defective sector\n"));
       PrintCLI(_("  --read-medium n        - read the whole medium up to n times\n"));
       PrintCLI(_("  --read-raw             - performs read in raw mode if possible\n"));
+      PrintCLI(_("  --resource-file p      - get resource file from given path\n"));
       PrintCLI(_("  --speed-warning n      - print warning if speed changes by more than n percent\n"));
       PrintCLI(_("  --spinup-delay n       - wait n seconds for drive to spin up\n"));
 
@@ -986,37 +934,45 @@ int main(int argc, char *argv[])
 	PrintCLI(_("  --copy-sector a,n,b,m - copy sector n from image a to sector m in image b\n"));
 	PrintCLI(_("  --erase sector    - erase the given sector\n"));
 	PrintCLI(_("  --erase n-m       - erase sectors n - m, inclusively\n"));
+	PrintCLI(_("  --fixed-speed-values - output fixed speed values for better output diffing\n"));
 	PrintCLI(_("  --marked-image n  - create image with n marked random sectors\n"));
 	PrintCLI(_("  --merge-images a,b  merge image a with b (a receives sectors from b)\n"));
-	PrintCLI(_("  --random-errors r,e seed image with (correctable) random errors\n"));
+	PrintCLI(_("  --random-errors e - seed image with (correctable) random errors\n"));
 	PrintCLI(_("  --random-image n  - create image with n sectors of random numbers\n"));
 	PrintCLI(_("  --random-seed n   - random seed for built-in random number generator\n"));
 	PrintCLI(_("  --raw-sector n    - shows hexdump of the given raw sector from medium in drive\n"));
 	PrintCLI(_("  --read-sector n   - shows hexdump of the given sector from medium in drive\n"));
 	PrintCLI(_("  --screen-shot     - useful for generating screen shots\n"));
 	PrintCLI(_("  --send-cdb arg    - executes given cdb at drive; kills system if used wrong\n"));
+	PrintCLI(_("  --set-version     - set program version for debugging purposes (dangerous!)\n"));
+	PrintCLI(_("  --show-header n   - assumes given sector is a ecc header and prints it\n"));
 	PrintCLI(_("  --show-sector n   - shows hexdump of the given sector in an image file\n"));
+	PrintCLI(_("  --sim-cd image    - simulate a SCSI-Level CD with contents supplied by the ISO image\n"));
 	PrintCLI(_("  --sim-defects n   - simulate n%% defective sectors on medium\n"));
 	PrintCLI(_("  --truncate n      - truncates image to n sectors\n")); 
 	PrintCLI(_("  --zero-unreadable - replace the \"unreadable sector\" markers with zeros\n\n"));
       }
 
-#ifdef WIN_CONSOLE
-      PrintCLI(_("NOTE: This is the Windows console version of dvdisaster.\n"
-		 "The version providing a graphical user interface is called\n"
-		 "dvdisaster-win.exe (also contained in this installation).\n\n"));
-#endif
       FreeClosure();
       exit(EXIT_FAILURE);
    }
 
    /* If no mode was selected at the command line, 
-      start the graphical user interface. 
-      Unless we are in Windows console mode where starting the GUI
-      won't work. */
+      start the graphical user interface. */
 
    if(mode == MODE_NONE)
    {  
+      /* We need to query devices in order to build
+	 the drop-down menu.*/
+
+      if(!devices_queried)
+      {  if(Closure->device)
+	    g_free(Closure->device);
+	 Closure->device = DefaultDevice();
+      }
+
+      /* Insert generic drive name for screen shot mode */
+
       if(Closure->screenShotMode)
       {  GPtrArray *a=Closure->deviceNames;
 	 int i;
@@ -1027,22 +983,11 @@ int main(int argc, char *argv[])
 	 }
       }
 
-
-      /* We need to query devices in order to build
-	 the drop-down menu.*/
-
-      if(!devices_queried)
-      {  if(Closure->device)
-	    g_free(Closure->device);
-	 Closure->device = DefaultDevice();
-      }
-
       Closure->guiMode = TRUE;
       ReadDotfile();
       CreateMainWindow(&argc, &argv);
    }
 
    FreeClosure();
-
    exit(exitCode);
 }
