@@ -1,25 +1,23 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2010 Carsten Gnoerlich.
- *  Project home page: http://www.dvdisaster.com
- *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
+ *  Copyright (C) 2004-2015 Carsten Gnoerlich.
  *
- *  The Reed-Solomon error correction draws a lot of inspiration - and even code -
- *  from Phil Karn's excellent Reed-Solomon library: http://www.ka9q.net/code/fec/
+ *  Email: carsten@dvdisaster.org  -or-  cgnoerlich@fsfe.org
+ *  Project homepage: http://www.dvdisaster.org
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of dvdisaster.
+ *
+ *  dvdisaster is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  dvdisaster is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA,
- *  or direct your browser at http://www.gnu.org.
+ *  along with dvdisaster. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "dvdisaster.h"
@@ -35,6 +33,8 @@ static void destroy(Method*);
 void register_rs01(void)
 {  Method *method = g_malloc0(sizeof(Method));
 
+   method->ckSumClosure = g_malloc0(sizeof(RS01CksumClosure));
+
    /*** Standard infomation and methods */ 
 
    strncpy(method->name, "RS01", 4);
@@ -46,7 +46,12 @@ void register_rs01(void)
 
    /*** Linkage to rs01-common.c */
 
-   method->recognizeEccFile = RS01Recognize;
+   method->recognizeEccFile  = RS01Recognize;
+   method->getCrcBuf         = RS01GetCrcBuf;
+   method->resetCksums       = RS01ResetCksums;
+   method->updateCksums      = RS01UpdateCksums;
+   method->finalizeCksums    = RS01FinalizeCksums;
+   method->expectedImageSize = RS01ExpectedImageSize;
 
    /*** Linkage to rs01-window.c */
 
@@ -74,6 +79,8 @@ void register_rs01(void)
 static void destroy(Method *method)
 {  RS01Widgets *wl = (RS01Widgets*)method->widgetList;
 
+   g_free(method->ckSumClosure);
+
    if(wl)
    {  if(wl->fixCurve) FreeCurve(wl->fixCurve);
 
