@@ -1,22 +1,23 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2010 Carsten Gnoerlich.
- *  Project home page: http://www.dvdisaster.com
- *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
+ *  Copyright (C) 2004-2015 Carsten Gnoerlich.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  Email: carsten@dvdisaster.org  -or-  cgnoerlich@fsfe.org
+ *  Project homepage: http://www.dvdisaster.org
+ *
+ *  This file is part of dvdisaster.
+ *
+ *  dvdisaster is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  dvdisaster is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA,
- *  or direct your browser at http://www.gnu.org.
+ *  along with dvdisaster. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "dvdisaster.h"
@@ -33,19 +34,17 @@ typedef struct
    RS03Layout *lay;
    GaloisTables *gt;
    ReedSolomonTables *rt;
+   Image *image;
    int earlyTermination;
    char *msg;
-   ImageInfo *ii;
-   LargeFile *eccFile;
    unsigned char *imgBlock[255];
-   gint64 *eccIdx[255];
 } fix_closure;
 
 static void fix_cleanup(gpointer data)
 {  fix_closure *fc = (fix_closure*)data;
    int i;
 
-   Closure->cleanupProc = NULL;
+   UnregisterCleanup();
 
    if(Closure->guiMode)
    {  if(fc->earlyTermination)
@@ -59,22 +58,14 @@ static void fix_cleanup(gpointer data)
    /** Clean up */
 
    if(fc->msg) g_free(fc->msg);
-   if(fc->ii) FreeImageInfo(fc->ii);
+   if(fc->image) CloseImage(fc->image);
 
    for(i=0; i<255; i++)
    {  if(fc->imgBlock[i])
 	 g_free(fc->imgBlock[i]); 
-
-      if(fc->eccIdx[i])
-	 g_free(fc->eccIdx[i]); 
-   }
-
-   if(fc->lay) 
-   {  if(fc->lay->target == ECC_FILE && fc->eccFile)
-	 LargeClose(fc->eccFile);
-      g_free(fc->lay);
    }
 
+   if(fc->lay) g_free(fc->lay);
    if(fc->gt) FreeGaloisTables(fc->gt);
    if(fc->rt) FreeReedSolomonTables(fc->rt);
 
@@ -88,25 +79,29 @@ static void fix_cleanup(gpointer data)
  * Expand a truncated image 
  */
 
-static void expand_image(ImageInfo *ii, gint64 new_size)
+static void expand_image(Image *image, EccHeader *eh, gint64 new_size)
 {  int last_percent, percent;
    gint64 sectors, new_sectors;
 
-   if(!LargeSeek(ii->file, ii->size))
+   if(!LargeSeek(image->file, image->file->size))
      Stop(_("Failed seeking to end of image: %s\n"), strerror(errno));
 
    last_percent = 0;
-   new_sectors = new_size - ii->sectors;
+   new_sectors = new_size - image->sectorSize;
+
    for(sectors = 0; sectors < new_sectors; sectors++)
    {  unsigned char buf[2048];
-      int n;
+      int length,n;
 
-      CreateMissingSector(buf, ii->sectors+sectors, 
-			  ii->mediumFP, FINGERPRINT_SECTOR, 
+      CreateMissingSector(buf, image->sectorSize+sectors, 
+			  image->imageFP, FINGERPRINT_SECTOR, 
 			  "RS03 fix placeholder");
 
-      n = LargeWrite(ii->file, buf, 2048);
-      if(n != 2048)
+      if(sectors != new_sectors-1) length = 2048;
+      else length = eh->inLast;  /* non-image file may be clipped */
+
+      n = LargeWrite(image->file, buf, length);
+      if(n != length)
 	Stop(_("Failed expanding the image: %s\n"), strerror(errno));
 
       percent = (100*sectors) / new_sectors;
@@ -130,23 +125,19 @@ static void expand_image(ImageInfo *ii, gint64 new_size)
  *** Test and fix the current image.
  ***/
 
-void RS03Fix(Method *self)
-{  RS03Widgets *wl = (RS03Widgets*)self->widgetList;
+void RS03Fix(Image *image)
+{  Method *self = FindMethod("RS03");
+   RS03Widgets *wl = (RS03Widgets*)self->widgetList;
    RS03Layout *lay;
    fix_closure *fc = g_malloc0(sizeof(fix_closure)); 
-   ImageInfo *ii = NULL;
-   LargeFile *eccfile;
    EccHeader *eh;
-#ifdef HAVE_BIG_ENDIAN
-   EccHeader *eh_swapped;
-#endif
    gint32 *gf_index_of;
    gint32 *gf_alpha_to;
    gint64 block_idx[255];
    gint64 s;
    guint32 *crc_buf, last_crc_sector1[512], last_crc_sector2[512];
    int nroots,ndata;
-   int crc_idx, ecc_idx;
+   int crc_idx;
    int crc_valid = TRUE;
    int cache_size, cache_sector, cache_offset;
    int erasure_count,erasure_list[255],erasure_map[255];
@@ -170,11 +161,14 @@ void RS03Fix(Method *self)
 
    /*** Register the cleanup procedure for GUI mode */
 
+   fc->image = image;
    fc->wl = wl;
    fc->earlyTermination = TRUE;
    RegisterCleanup(_("Repairing of image aborted"), fix_cleanup, fc);
 
-   eh  = self->lastEh;  /* will always be present */
+   if(image->eccFileHeader)
+        eh = image->eccFileHeader;
+   else eh = image->eccHeader;
 
    /*** Open the image file */
 
@@ -183,20 +177,13 @@ void RS03Fix(Method *self)
 		  _("<big>Repairing the image.</big>\n<i>%s</i>"),
 		  _("Opening files..."));
 
-   ii  = fc->ii = OpenImageFile(eh, WRITEABLE_IMAGE);
-
    /* Calculate the layout and optinally open thee ecc file */
 
    if(eh->methodFlags[0] & MFLAG_ECC_FILE)
-   {  lay = fc->lay = CalcRS03Layout(uchar_to_gint64(eh->sectors), eh, ECC_FILE); 
-      eccfile = fc->eccFile = LargeOpen(Closure->eccName, O_RDWR, IMG_PERMS);
-
-      if(!eccfile)  /* Failing here is unlikely since caller could open it */
-	 Stop("Could not open %s: %s",Closure->eccName, strerror(errno));
+   {  lay = fc->lay = CalcRS03Layout(image, ECC_FILE); 
    }
    else 
-   {  lay = fc->lay = CalcRS03Layout(uchar_to_gint64(eh->sectors), eh, ECC_IMAGE); 
-      eccfile = ii->file;
+   {  lay = fc->lay = CalcRS03Layout(image, ECC_IMAGE); 
    }
 
    ndata  = lay->ndata;
@@ -209,39 +196,84 @@ void RS03Fix(Method *self)
    gf_index_of = fc->gt->indexOf;
    gf_alpha_to = fc->gt->alphaTo;
 
-   /*** Expand a truncated image with "dead sector" markers */
+   /*** Expand a truncated image with "dead sector" markers.
+        If the images have the same number of sectors but a 
+        different number of bytes in the last sector, 
+        expand_image() is not called here as the missing bytes
+        are honoured in the RS03ReadSectors() functions and
+        the error correction will expand the image by writing
+        out the correct number of bytes. */
 
    if(eh->methodFlags[0] & MFLAG_ECC_FILE)
         expected_sectors = lay->dataSectors;
    else expected_sectors = lay->totalSectors;
 
-   if(ii->sectors < expected_sectors)
-     expand_image(ii, expected_sectors);
+   if(image->sectorSize < expected_sectors)
+     expand_image(image, eh, expected_sectors);
 
    /*** Announce what we are going to do */
 
-   if(eh->methodFlags[0] & MFLAG_ECC_FILE)
-      msg = g_strdup_printf(_("Error correction file using Method RS03, %d roots, %4.1f%% redundancy."),
-			    eh->eccBytes, 
-			    ((double)eh->eccBytes*100.0)/(double)eh->dataBytes);
-   else
-      msg = g_strdup_printf(_("Image contains error correction data: Method RS03, %d roots, %4.1f%% redundancy."),
-			    eh->eccBytes, 
-			    ((double)eh->eccBytes*100.0)/(double)eh->dataBytes);
-
    if(Closure->guiMode)
-   {  SetLabelText(GTK_LABEL(wl->fixHeadline),
+   {  if(eh->methodFlags[0] & MFLAG_ECC_FILE)
+        msg = g_strdup_printf(_("Error correction file using Method RS03, %d roots, %4.1f%% redundancy."),
+			      eh->eccBytes, 
+			      ((double)eh->eccBytes*100.0)/(double)eh->dataBytes);
+      else
+	msg = g_strdup_printf(_("Image contains error correction data: Method RS03, %d roots, %4.1f%% redundancy."),
+			      eh->eccBytes, 
+			      ((double)eh->eccBytes*100.0)/(double)eh->dataBytes);
+
+      SetLabelText(GTK_LABEL(wl->fixHeadline),
 		  _("<big>Repairing the image.</big>\n<i>%s</i>"), msg);
       RS03SetFixMaxValues(wl, eh->dataBytes, eh->eccBytes, expected_sectors);
       g_free(msg);
    }    
 
-   PrintLog(_("\nFix mode: Repairable sectors will be fixed in the image.\n"));
+   PrintLog(_("\nFix mode(%s): Repairable sectors will be fixed in the image.\n"),
+	    eh->methodFlags[0] & MFLAG_ECC_FILE ? "RS03f" : "RS03i");
 
-   /*** Truncate an image with trailing garbage */
+   /*** Image is a few bytes too long */
 
-   if(ii->sectors > expected_sectors)
-   { gint64 diff = ii->sectors - expected_sectors;
+   if(image->sectorSize == expected_sectors && image->inLast > eh->inLast)
+   {  int difference = image->inLast - eh->inLast;
+      guint64 expected_image_size = 2048*(expected_sectors-1)+eh->inLast;
+
+      if(Closure->guiMode)
+      {  int answer = ModalDialog(GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL, NULL,
+				  _("The image file is %d bytes longer than noted\n"
+				    "in the ecc file. Shall the superfluous bytes\n"
+				    "be removed from the image file?\n"),
+				    difference);
+
+	 if(!answer)
+	 {  SwitchAndSetFootline(fc->wl->fixNotebook, 1,
+				 fc->wl->fixFootline,
+				 _("<span %s>Aborted by user request!</span>"),
+				 Closure->redMarkup); 
+	    fc->earlyTermination = FALSE;  /* suppress respective error message */
+	    goto terminate;
+	 }
+      }
+
+      if(!Closure->guiMode && !Closure->truncate)
+        Stop(_("The image file is %d bytes longer than noted\n"
+	       "in the ecc file.\n"
+               "Add the --truncate option to the program call\n"
+	       "to have the superfluous sectors removed."),
+	     difference);
+      
+      if(!LargeTruncate(image->file, expected_image_size))
+	Stop(_("Could not truncate %s: %s\n"),Closure->imageName,strerror(errno));
+
+      PrintLog(_("Image has been truncated by %d bytes.\n"), difference);
+      image->inLast = eh->inLast;
+   }
+
+   /*** Truncate an image with trailing garbage (more than one sector) */
+
+   if(image->sectorSize > expected_sectors)
+   { gint64 diff = image->sectorSize - expected_sectors;
+     guint64 expected_image_size = 2048*(expected_sectors-1)+eh->inLast;
      char *trans = _("The image file is %lld sectors longer as noted in the\n"
 		     "ecc data. This might simply be zero padding, but could\n"
 		     "also mean that the image was manipulated after appending\n"
@@ -263,9 +295,9 @@ void RS03Fix(Method *self)
 	   goto terminate;
 	}
 
-        ii->sectors -= diff;
+        image->sectorSize -= diff;
 
-        if(!LargeTruncate(ii->file, (gint64)(2048*ii->sectors)))
+        if(!LargeTruncate(image->file, (gint64)expected_image_size))
 	  Stop(_("Could not truncate %s: %s\n"),Closure->imageName,strerror(errno));
      }
      
@@ -284,9 +316,9 @@ void RS03Fix(Method *self)
 	  goto terminate;
        }
 
-       ii->sectors -= diff;
+       image->sectorSize -= diff;
 
-       if(!LargeTruncate(ii->file, (gint64)(2048*ii->sectors)))
+       if(!LargeTruncate(image->file, (gint64)expected_image_size))
 	 Stop(_("Could not truncate %s: %s\n"),Closure->imageName,strerror(errno));
 
        PrintLog(_("Image has been truncated by %lld sectors.\n"), diff);
@@ -299,9 +331,9 @@ void RS03Fix(Method *self)
 		_("Add the --truncate option to the program call\n"
 		  "to have the superfluous sectors removed."));
 
-         ii->sectors -= diff;
+         image->sectorSize -= diff;
 
-	 if(!LargeTruncate(ii->file, (gint64)(2048*ii->sectors)))
+	 if(!LargeTruncate(image->file, (gint64)expected_image_size))
 	   Stop(_("Could not truncate %s: %s\n"),Closure->imageName,strerror(errno));
 
 	 PrintLog(_("Image has been truncated by %lld sectors.\n"), diff);
@@ -317,29 +349,24 @@ void RS03Fix(Method *self)
 	A portion of cache_size sectors is read ahead from each layer,
 	giving a total cache size of 255*cache_size. */
 
-   cache_size = 2*Closure->cacheMB;  /* ndata+nroots=255 medium sectors are approx. 0.5MB */
+   cache_size = 2*Closure->cacheMiB;  /* ndata+nroots=255 medium sectors are approx. 0.5MiB */
 
    for(i=0; i<255; i++)
       fc->imgBlock[i] = g_malloc(cache_size*2048);
 
-   for(i=0; i<nroots; i++)
-      fc->eccIdx[i] = g_malloc(cache_size*sizeof(gint64));
-
    /*** Setup the block counters for mapping medium sectors to ecc blocks.
 	We begin at the first ecc block (0) */
 
    for(s=0, i=0; i<lay->ndata; s+=lay->sectorsPerLayer, i++)
      block_idx[i] = s;
 
-   ecc_idx = 0;
-
    cache_sector = cache_size;  /* forces instant reload of imgBlock cache */
    cache_offset = 2048*cache_sector;
 
    /*** CRC sums for the first ecc block are stored in the last CRC sector.
 	Error handling is done later when this sector is actually used. */
 
-   RS03ReadSectors(eccfile, lay, 
+   RS03ReadSectors(image, lay, 
 		   (unsigned char*)last_crc_sector2, 
 		   lay->ndata-1, lay->sectorsPerLayer-1, 1, RS03_READ_CRC);
 
@@ -353,10 +380,11 @@ void RS03Fix(Method *self)
      /* See if user hit the Stop button */
 
      if(Closure->stopActions) 
-     {   SwitchAndSetFootline(fc->wl->fixNotebook, 1,
-			      fc->wl->fixFootline,
-			      _("<span %s>Aborted by user request!</span>"),
-			      Closure->redMarkup); 
+     {   if(Closure->stopActions == STOP_CURRENT_ACTION) /* suppress memleak warning when closing window */
+	   SwitchAndSetFootline(fc->wl->fixNotebook, 1,
+				fc->wl->fixFootline,
+				_("<span %s>Aborted by user request!</span>"),
+				Closure->redMarkup); 
          fc->earlyTermination = FALSE;  /* suppress respective error message */
 	 goto terminate;
      }
@@ -372,13 +400,13 @@ void RS03Fix(Method *self)
 
         for(i=0; i<ndata-1; i++)
         {  
-	   RS03ReadSectors(ii->file, lay, fc->imgBlock[i], i, ecc_idx, 
+	   RS03ReadSectors(image, lay, fc->imgBlock[i], i, s, 
 			   cache_size, RS03_READ_DATA);
 	}
 
 	/* Read from the CRC layer */
 
-	RS03ReadSectors(eccfile, lay, fc->imgBlock[ndata-1], ndata-1, ecc_idx,
+	RS03ReadSectors(image, lay, fc->imgBlock[ndata-1], ndata-1, s,
 			cache_size, RS03_READ_CRC);
 
 	/* Keep a copy of the last CRC sector for the next pass */
@@ -389,13 +417,8 @@ void RS03Fix(Method *self)
 
         for(i=0; i<nroots; i++)
         {  
-	   RS03ReadSectors(eccfile, lay, fc->imgBlock[i+ndata], i+ndata, ecc_idx,
+	   RS03ReadSectors(image, lay, fc->imgBlock[i+ndata], i+ndata, s,
 			   cache_size, RS03_READ_ECC);
-
-	   /* Remember virtual (= augmented image sectors) in ecc file case */
-
-	   for(j=0; j<cache_size; j++) 
-	      fc->eccIdx[i][j] = (i+ndata)*lay->sectorsPerLayer + ecc_idx+j;
 	}
 
         cache_sector = cache_offset = 0;
@@ -417,7 +440,7 @@ void RS03Fix(Method *self)
      {  int err;
 
 	crc_buf   = (guint32*)(fc->imgBlock[ndata-1]+cache_offset-2048);
-	/* fixme: replace 0 with real CRC sector number */
+
 	err = CheckForMissingSector((unsigned char*)crc_buf, 
 				    block_idx[ndata-1],
 				    eh->mediumFP, eh->fpSector);
@@ -467,9 +490,9 @@ void RS03Fix(Method *self)
 
      for(i=lay->ndata; i<GF_FIELDMAX; i++)
      {  int err = CheckForMissingSector(fc->imgBlock[i]+cache_offset,
-					fc->eccIdx[i-ndata][cache_sector],
+					RS03SectorIndex(lay, i, s),
 					eh->mediumFP, eh->fpSector);
-	//FIXME: wrong sector number in ecc files
+
 	if(err)
 	{  erasure_map[i] = 1;
 	   erasure_list[erasure_count++] = i;
@@ -484,15 +507,20 @@ void RS03Fix(Method *self)
 
      if(erasure_count>lay->nroots)   /* uncorrectable */
      {  if(!Closure->guiMode)
-	{  PrintCLI(_("* Ecc block %lld: %3d unrepairable sectors: "), s, erasure_count);
+	{  int sep_printed = 0;
+
+           PrintCLI(_("* Ecc block %lld: %3d unrepairable sectors: "), s, erasure_count);
 
 	   for(i=0; i<erasure_count; i++)
-	   {  gint64 loc = erasure_list[i];
-	      PrintCLI("%lld ", 
-		       loc < ndata ? block_idx[loc] : fc->eccIdx[loc-ndata][cache_sector]);
+	   {  /* sector counting wraps to 0 for ecc files after the data layer */
+              if(eh->methodFlags[0] & MFLAG_ECC_FILE && erasure_list[i] >= ndata-1 && ! sep_printed)
+	      {  PrintCLI("; ecc file: ");
+                 sep_printed = 1;
+              }
+              PrintCLI("%lld ", RS03SectorIndex(lay, erasure_list[i], s));
 	   }
 	   PrintCLI("\n");
-	}
+     }
 
 	uncorrected += erasure_count;
 	goto skip;
@@ -641,14 +669,18 @@ void RS03Fix(Method *self)
 	/* deg(lambda) unequal to number of roots => uncorrectable error detected */
 
 	if(deg_lambda != count)
-	{  PrintLog("Decoder problem (%d != %d) for %d sectors: ", deg_lambda, count, erasure_count);
+	{  int sep_printed = 0;
+           PrintLog("Decoder problem (%d != %d) for %d sectors: ", deg_lambda, count, erasure_count);
 
 	   for(i=0; i<erasure_count; i++)
-	   {  gint64 loc = erasure_list[i];
-	      PrintLog("%lld ", 
-		       loc < ndata ? block_idx[loc] : fc->eccIdx[loc-ndata][cache_sector]);
+	   {  /* sector counting wraps to 0 for ecc files after the data layer */
+              if(eh->methodFlags[0] & MFLAG_ECC_FILE && erasure_list[i] >= ndata-1 && ! sep_printed)
+	      {  PrintCLI(_("; ecc file: "));
+                 sep_printed = 1;
+              }
+              PrintCLI("%lld ", RS03SectorIndex(lay, erasure_list[i], s));
 	   }
-	   PrintLog("\n");
+	   PrintCLI("\n");
 	   uncorrected += erasure_count;
 	   goto skip;
 	}
@@ -695,30 +727,32 @@ void RS03Fix(Method *self)
 
 	   if(num1 != 0)
 	   {  int location = loc[j];
-		
+
 	      if(erasure_map[location] != 1)  /* erasure came from CRC error */
 	      {  int old = fc->imgBlock[location][offset];
 		 int new = old ^ gf_alpha_to[mod_fieldmax(gf_index_of[num1] + gf_index_of[num2] + GF_FIELDMAX - gf_index_of[den])];
-		 char *msg;
+		 char *msg, *type;
 		 gint64 sector;
 
 		 if(erasure_map[location] == 3)  /* erasure came from CRC error */
-		 {  msg = _("-> CRC-predicted error in sector %lld at byte %4d (value %02x '%c', expected %02x '%c')\n");
+		 {  msg = _("-> CRC-predicted error in sector %lld%s at byte %4d (value %02x '%c', expected %02x '%c')\n");
 		 }
 		 else
-		 {  msg = _("-> Non-predicted error in sector %lld at byte %4d (value %02x '%c', expected %02x '%c')\n");
+		 {  msg = _("-> Non-predicted error in sector %lld%s at byte %4d (value %02x '%c', expected %02x '%c')\n");
 		    if(erasure_map[location] == 0) /* remember error location */
 		    {  erasure_map[location] = 7;
 		       error_count++;  
 		    }
 		 }
 
-		 if(location < ndata)
-		      sector = block_idx[location];
-		 else sector = fc->eccIdx[location-ndata][cache_sector];
-
+		 sector = RS03SectorIndex(lay, location, s);
+		 if(eh->methodFlags[0] & MFLAG_ECC_FILE && location >= ndata-1)
+		   type="(ecc)";
+		 else
+		   type="";
+		 
 		 PrintCLI(msg,
-			  sector, bi, 
+	                  sector, type, bi, 
 			  old, isprint(old) ? old : '.',
 			  new, isprint(new) ? new : '.');
 	      }
@@ -734,7 +768,8 @@ void RS03Fix(Method *self)
      erasure_count += error_count;  /* total errors encountered */
 
      if(erasure_count)
-     {  PrintCLI(_("  %3d repaired sectors: "), erasure_count);
+     {  int sep_printed = 0;
+        PrintCLI(_("  %3d repaired sectors: "), erasure_count);
 
         for(i=0; i<255; i++)
         {  gint64 sec;
@@ -758,10 +793,15 @@ void RS03Fix(Method *self)
 	        break;
 	   }
 
-	   if(i < ndata) {  data_corr++; sec = block_idx[i]; }
-	   else          {  ecc_corr++;  sec = fc->eccIdx[i-ndata][cache_sector]; }
+	   sec = RS03SectorIndex(lay, i, s);
+	   if(i < ndata) {  data_corr++;  }
+	   else          {  ecc_corr++;   }
 	   corrected++;
 
+	   if(eh->methodFlags[0] & MFLAG_ECC_FILE && i >= ndata-1 && ! sep_printed)
+	   {  PrintCLI(_("; ecc file: "));
+              sep_printed = 1;
+           }
 	   PrintCLI("%lld%c ", sec, type);
 
 	   /* Write the recovered sector */
@@ -772,37 +812,32 @@ void RS03Fix(Method *self)
 	   /* Write back into the image */
 
 	   if(   lay->target == ECC_IMAGE 
-	      || sec < lay->dataSectors)
+	      || i < ndata-1)
 	   {
-	      if(!LargeSeek(ii->file, (gint64)(2048*sec)))
+	      if(!LargeSeek(image->file, (gint64)(2048*sec)))
 		 Stop(_("Failed seeking to sector %lld in image [%s]: %s"),
 		      sec, "FW", strerror(errno));
 
-	      n = LargeWrite(ii->file, cache_offset+fc->imgBlock[i], length);
+	      n = LargeWrite(image->file, cache_offset+fc->imgBlock[i], length);
 	      if(n != length)
 		 Stop(_("could not write medium sector %lld:\n%s"), sec, strerror(errno));
 	   }
 
-	   /* Write back into the error correction file.
+	   /* Write back into the error correction file
+	      (for the CRC and ECC portion of the ecc block).
 	      Note that "sec" contains the virtual adresses as
 	      if we were processing an augmented image. */
 
-	   if(   lay->target == ECC_FILE
-	      && sec >= lay->firstCrcPos)  //FIXME: correctness?
-	   {  gint64 first_crc_pos = (lay->ndata-1)*lay->sectorsPerLayer;
-
-	      if(sec >= first_crc_pos)
-	      {  gint64 real_sec = 2+sec-first_crc_pos;
-
-		 if(!LargeSeek(eccfile, (gint64)(2048*real_sec)))
+	   if(lay->target == ECC_FILE && i >= ndata-1)
+	   {  
+              if(!LargeSeek(image->eccFile, (gint64)(2048*sec)))
 		 Stop(_("Failed seeking to sector %lld in ecc file [%s]: %s"),
-		      real_sec, "FW", strerror(errno));
+		      sec, "FW", strerror(errno));
 
-		 n = LargeWrite(eccfile, cache_offset+fc->imgBlock[i], 2048);
+		 n = LargeWrite(image->eccFile, cache_offset+fc->imgBlock[i], 2048);
 		 if(n != 2048)
 		    Stop(_("could not write ecc file sector %lld:\n%s"),
-			 real_sec, strerror(errno));
-	      }
+			 sec, strerror(errno));
 	   }
 	}
 	PrintCLI("\n");
@@ -846,8 +881,6 @@ skip:
 
      for(i=0; i<lay->ndata; i++)
 	block_idx[i]++;
-
-     ecc_idx++;
    }
 
    /*** Print results */
