@@ -1,22 +1,23 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2010 Carsten Gnoerlich.
- *  Project home page: http://www.dvdisaster.com
- *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
+ *  Copyright (C) 2004-2015 Carsten Gnoerlich.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  Email: carsten@dvdisaster.org  -or-  cgnoerlich@fsfe.org
+ *  Project homepage: http://www.dvdisaster.org
+ *
+ *  This file is part of dvdisaster.
+ *
+ *  dvdisaster is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  dvdisaster is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA,
- *  or direct your browser at http://www.gnu.org.
+ *  along with dvdisaster. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "dvdisaster.h"
@@ -89,6 +90,78 @@ static void eccmethod_cb(GtkWidget *widget, gpointer data)
    }
 }
 
+/*
+ * Codec type selection 
+ */
+
+static void encoding_alg_cb(GtkWidget *widget, gpointer data)
+{  RS03Widgets *wl = (RS03Widgets*)data;
+   int state  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+   if(!state)  /* only track changes to activate state */
+     return;
+
+   if(widget == wl->eaRadio1A || widget == wl->eaRadio1B)
+   {  Closure->encodingAlgorithm = ENCODING_ALG_32BIT;
+
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio1A), TRUE); 
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio1B), TRUE); 
+   }
+
+   if(widget == wl->eaRadio2A || widget == wl->eaRadio2B)
+   {  Closure->encodingAlgorithm = ENCODING_ALG_64BIT;
+
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio2A), TRUE); 
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio2B), TRUE); 
+   }
+
+   if(widget == wl->eaRadio3A || widget == wl->eaRadio3B)
+   { 
+#ifdef HAVE_SSE2 
+      Closure->encodingAlgorithm = ENCODING_ALG_SSE2;
+#endif
+#ifdef HAVE_ALTIVEC 
+      Closure->encodingAlgorithm = ENCODING_ALG_ALTIVEC;
+#endif
+
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio3A), TRUE); 
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio3B), TRUE); 
+   }
+
+   if(widget == wl->eaRadio4A || widget == wl->eaRadio4B)
+   {  Closure->encodingAlgorithm = ENCODING_ALG_DEFAULT;
+
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio4A), TRUE); 
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->eaRadio4B), TRUE); 
+   }
+}
+
+/*
+ * I/O strategy selection
+ */
+
+static void io_strategy_cb(GtkWidget *widget, gpointer data)
+{  RS03Widgets *wl = (RS03Widgets*)data;
+   int state  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+   if(!state)  /* only track changes to activate state */
+     return;
+
+   if(widget == wl->ioRadio1A || widget == wl->ioRadio1B)
+   {  Closure->encodingIOStrategy = IO_STRATEGY_READWRITE;
+
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->ioRadio1A), TRUE); 
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->ioRadio1B), TRUE); 
+   }
+
+   if(widget == wl->ioRadio2A || widget == wl->ioRadio2B)
+   {  Closure->encodingIOStrategy = IO_STRATEGY_MMAP;
+
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->ioRadio2A), TRUE); 
+      activate_toggle_button(GTK_TOGGLE_BUTTON(wl->ioRadio2B), TRUE); 
+   }
+}
+
 /*
  * Setting the notebook page does not work at creation time.
  */
@@ -396,65 +469,18 @@ void ReadRS03Preferences(Method *method)
 #endif
 }
 
-static gboolean wrapper_fix_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
-{  int *last_width = (int*)data;
-   int label_width = widget->allocation.width;
-   
-   if(*last_width == label_width)  /* short circuit expose events */ 
-      return FALSE;                /* without size changes */
-
-   *last_width = label_width;
-
-   /* This is a hack. We feed the label its own allocation to make it redraw.
-      Note that we subtract 4 or else the window would never shrink again. */
-
-   if(label_width<0 || label_width>200)
-      gtk_widget_set_size_request(widget, label_width-4, -1);
-
-   return FALSE;
-}
-
 /*
  * Create our preferences page
  */
 
 void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
-{  static int warning_width;
-   RS03Widgets *wl = (RS03Widgets*)method->widgetList;
+{  RS03Widgets *wl = (RS03Widgets*)method->widgetList;
    GtkWidget *frame, *hbox, *vbox, *lab, *scale, *spin, *radio;
    LabelWithOnlineHelp *lwoh;
    unsigned int index;
    char *text;
    int i;
 
-   /*** Experimental release warning */
-
-   frame = gtk_frame_new(NULL);
-   gtk_widget_modify_fg(frame, GTK_STATE_NORMAL, Closure->redText);
-   gtk_widget_modify_bg(frame, GTK_STATE_NORMAL, Closure->redText);
-   gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
-   gtk_box_pack_start(GTK_BOX(parent), frame, FALSE, FALSE, 0);
-
-   vbox = gtk_vbox_new(FALSE, 10);
-   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
-   gtk_container_add(GTK_CONTAINER(frame), vbox);
-
-   lab = gtk_label_new(NULL);
-   text = g_locale_to_utf8(_("<b>Warning</b>: This codec is experimental and for evaluation only. "
-			     "It is not yet fully implemented and does contain bugs. "
-			     "Future dvdisaster versions may not be compatible with it. "
-			     "<i>Do not yet use this codec for any archival purposes.</i>"),
-			   -1, NULL, NULL, NULL);
-   gtk_label_set_markup(GTK_LABEL(lab), text);
-   g_free(text);
-
-   gtk_misc_set_alignment(GTK_MISC(lab), 0.0, 0.0);
-   gtk_label_set_line_wrap(GTK_LABEL(lab), TRUE);
-   g_signal_connect(lab, "expose_event", G_CALLBACK(wrapper_fix_cb), &warning_width);
-
-   //   gtk_container_add(GTK_CONTAINER(frame), lab);
-   gtk_box_pack_start(GTK_BOX(vbox), lab, FALSE, FALSE, 0);
-
    /*** Target for error correction data */
 
    frame = gtk_frame_new(_utf("Error correction data storage"));
@@ -683,7 +709,7 @@ void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
       gtk_entry_set_width_chars(GTK_ENTRY(spin), 8);
       gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
 
-      lab = gtk_label_new(_utf("MB for error correction data"));
+      lab = gtk_label_new(_utf("MiB for error correction data"));
       gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);
       gtk_widget_set_sensitive(spin, FALSE);
       gtk_widget_set_sensitive(lab, FALSE);
@@ -701,7 +727,7 @@ void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
    }
 
    AddHelpParagraph(lwoh, _("<b>Space-delimited redundancy</b>\n\n"
-			    "Specifies the maximum size of the error correction file in MB. "
+			    "Specifies the maximum size of the error correction file in MiB. "
 			    "dvdisaster will choose a suitable redundancy setting so that "
 			    "the overall size of the error correction file does not exceed "
 			    "the given limit.\n\n"
@@ -741,13 +767,19 @@ void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
       }
    }
 
-   /*** Prefetch sectors */
+   /*** IO parameters */
 
-   frame = gtk_frame_new(_utf("Sector prefetch"));
+   /* Prefetch sectors */
+
+   frame = gtk_frame_new(_utf("I/O parameters"));
    gtk_box_pack_start(GTK_BOX(parent), frame, FALSE, FALSE, 0);
 
+   vbox = gtk_vbox_new(FALSE, 10);
+   gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+   gtk_container_add(GTK_CONTAINER(frame), vbox);
+
    text = g_strdup_printf(_("%d sectors"), Closure->prefetchSectors);
-   lwoh = CreateLabelWithOnlineHelp(_("Sector prefetching"), text);
+   lwoh = CreateLabelWithOnlineHelp(_("Sector preloading"), text);
    RegisterPreferencesHelpWindow(lwoh);
    g_free(text);
 
@@ -759,7 +791,7 @@ void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
    {  GtkWidget *hbox = gtk_hbox_new(FALSE, 4);
       int n_entries = sizeof(prefetch_size)/sizeof(int);
 
-      lab = gtk_label_new(_utf("Prefetch"));
+      lab = gtk_label_new(_utf("Preload"));
       gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);
 
       for(index = 0; index < n_entries; index++)
@@ -776,9 +808,8 @@ void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
 
       if(!i)
       {  wl->prefetchScaleA = scale; 
-	 gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
 	 gtk_box_pack_start(GTK_BOX(hbox), lwoh->linkBox, FALSE, FALSE, 0);
-	 gtk_container_add(GTK_CONTAINER(frame), hbox);
+	 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
       }
       else
       {  wl->prefetchScaleB = scale; 
@@ -794,8 +825,67 @@ void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
 			    "used for the image and error correction files.\n"
 			    "Use small preload values for systems with low latency "
 			    "and seek time, e.g. SSDs. For magnetic hard disks "
-			    "performace may be better using larger preload values.\n\n"
-			    "A preload value of n will used approx. n MB of RAM."));
+			    "performance may be better using larger preload values.\n\n"
+			    "A preload value of n will used approx. n MiB of RAM."));
+
+   /*** IO strategy */
+
+   lwoh = CreateLabelWithOnlineHelp(_("I/O strategy"), 
+				    _("I/O strategy: "));
+   RegisterPreferencesHelpWindow(lwoh);
+
+   for(i=0; i<2; i++)
+   {  GtkWidget *hbox = gtk_hbox_new(FALSE, 4);
+      GtkWidget *radio1, *radio2;
+
+      gtk_box_pack_start(GTK_BOX(hbox), i ? lwoh->normalLabel : lwoh->linkBox, FALSE, FALSE, 0);
+
+      radio1 = gtk_radio_button_new(NULL);
+      g_signal_connect(G_OBJECT(radio1), "toggled", G_CALLBACK(io_strategy_cb), (gpointer)wl);
+      gtk_box_pack_start(GTK_BOX(hbox), radio1, FALSE, FALSE, 0);
+      lab = gtk_label_new(_utf("read/write"));
+      gtk_container_add(GTK_CONTAINER(radio1), lab);
+
+      radio2 = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(radio1));
+      g_signal_connect(G_OBJECT(radio2), "toggled", G_CALLBACK(io_strategy_cb), (gpointer)wl);
+      gtk_box_pack_start(GTK_BOX(hbox), radio2, FALSE, FALSE, 0);
+      lab = gtk_label_new(_utf("memory mapped"));
+      gtk_container_add(GTK_CONTAINER(radio2), lab);
+
+      switch(Closure->encodingIOStrategy)
+      {  case IO_STRATEGY_READWRITE: activate_toggle_button(GTK_TOGGLE_BUTTON(radio1), TRUE); break;
+         case IO_STRATEGY_MMAP:      activate_toggle_button(GTK_TOGGLE_BUTTON(radio2), TRUE); break;
+      }
+
+      if(!i)
+      {  wl->ioRadio1A = radio1;
+	 wl->ioRadio2A = radio2;
+	 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+      }
+      else  
+      {  wl->ioRadio1B = radio1;
+	 wl->ioRadio2B = radio2;
+	 AddHelpWidget(lwoh, hbox);
+      }
+   }
+
+   AddHelpParagraph(lwoh, _("<b>I/O strategy</b>\n\n"
+     "This option controls how dvdisaster performs its disk I/O while creating error "
+     "correction data. Try both options and see which performs best on your hardware "
+     "setting.\n\n" 
+     "The <b>read/write</b> option activates dvdisaster's own I/O scheduler "
+     "which reads and writes image data using normal file I/O. The advantage of this "
+     "scheme is that dvdisaster knows exactly which data needs to be cached and preloaded; "
+     "the disadvantage is that all data needs to be copied between the kernel and "
+     "dvdisaster's own buffers. Usually, this I/O scheme works best on slow storage "
+     "with high latency and seek times; e.g. on all storage involving spinning platters.\n\n"
+     "The <b>memory mapped</b> option uses the kernel's memory mapping scheme for direct access "
+     "to the image file. This has the advantage of minimal overhead, but may be adversely "
+     "affected by poor caching and preloading decisions made by the kernel (since the kernel does not "
+     "know what dvdisaster is going to do with the data). This scheme "
+     "performs well when encoding in a RAM-based file system (such as /dev/shm on GNU/Linux) "
+     "and on very fast media with low latency such as SSDs."
+ 			    ));
 
    /*** Number of threads */
 
@@ -855,4 +945,92 @@ void CreateRS03PrefsPage(Method *method, GtkWidget *parent)
 			    "is more limiting than raw CPU power. When using "
 			    "4 cores or more, memory bandwidth may also affect "
 			    "performance."));
+
+   /*** Codec type */
+
+   frame = gtk_frame_new(_utf("Encoding algorithm"));
+   gtk_box_pack_start(GTK_BOX(parent), frame, FALSE, FALSE, 0);
+
+   vbox = gtk_vbox_new(FALSE, 10);
+   gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+   gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+   lwoh = CreateLabelWithOnlineHelp(_("Encoding algorithm"), 
+				    _("Use: "));
+   RegisterPreferencesHelpWindow(lwoh);
+
+   for(i=0; i<2; i++)
+   {  GtkWidget *hbox = gtk_hbox_new(FALSE, 4);
+      GtkWidget *radio1, *radio2, *radio3=NULL, *radio4;
+
+      gtk_box_pack_start(GTK_BOX(hbox), i ? lwoh->normalLabel : lwoh->linkBox, FALSE, FALSE, 0);
+
+      radio1 = gtk_radio_button_new(NULL);
+      g_signal_connect(G_OBJECT(radio1), "toggled", G_CALLBACK(encoding_alg_cb), (gpointer)wl);
+      gtk_box_pack_start(GTK_BOX(hbox), radio1, FALSE, FALSE, 0);
+      lab = gtk_label_new(_utf("32bit"));
+      gtk_container_add(GTK_CONTAINER(radio1), lab);
+
+      radio2 = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(radio1));
+      g_signal_connect(G_OBJECT(radio2), "toggled", G_CALLBACK(encoding_alg_cb), (gpointer)wl);
+      gtk_box_pack_start(GTK_BOX(hbox), radio2, FALSE, FALSE, 0);
+      lab = gtk_label_new(_utf("64bit"));
+      gtk_container_add(GTK_CONTAINER(radio2), lab);
+
+      if(Closure->useSSE2)
+      {  radio3 = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(radio2));
+	 g_signal_connect(G_OBJECT(radio3), "toggled", G_CALLBACK(encoding_alg_cb), (gpointer)wl);
+	 gtk_box_pack_start(GTK_BOX(hbox), radio3, FALSE, FALSE, 0);
+	 lab = gtk_label_new(_utf("SSE2"));
+	 gtk_container_add(GTK_CONTAINER(radio3), lab);
+      }
+      if(Closure->useAltiVec)
+      {  radio3 = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(radio2));
+	 g_signal_connect(G_OBJECT(radio3), "toggled", G_CALLBACK(encoding_alg_cb), (gpointer)wl);
+	 gtk_box_pack_start(GTK_BOX(hbox), radio3, FALSE, FALSE, 0);
+	 lab = gtk_label_new(_utf("AltiVec"));
+	 gtk_container_add(GTK_CONTAINER(radio3), lab);
+      }
+
+      radio4 = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(radio2));
+      g_signal_connect(G_OBJECT(radio4), "toggled", G_CALLBACK(encoding_alg_cb), (gpointer)wl);
+      gtk_box_pack_start(GTK_BOX(hbox), radio4, FALSE, FALSE, 0);
+      lab = gtk_label_new(_utf("auto"));
+      gtk_container_add(GTK_CONTAINER(radio4), lab);
+
+      switch(Closure->encodingAlgorithm)
+      {  case ENCODING_ALG_DEFAULT: activate_toggle_button(GTK_TOGGLE_BUTTON(radio4), TRUE); break;
+         case ENCODING_ALG_32BIT:   activate_toggle_button(GTK_TOGGLE_BUTTON(radio1), TRUE); break;
+         case ENCODING_ALG_64BIT:   activate_toggle_button(GTK_TOGGLE_BUTTON(radio2), TRUE); break;
+         case ENCODING_ALG_SSE2:    
+         case ENCODING_ALG_ALTIVEC: activate_toggle_button(GTK_TOGGLE_BUTTON(radio3), TRUE); break;
+      }
+
+      if(!i)
+      {  wl->eaRadio1A = radio1;
+	 wl->eaRadio2A = radio2;
+	 wl->eaRadio3A = radio3;
+	 wl->eaRadio4A = radio4;
+	 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+      }
+      else  
+      {  wl->eaRadio1B = radio1;
+	 wl->eaRadio2B = radio2;
+	 wl->eaRadio3B = radio3;
+	 wl->eaRadio4B = radio4;
+	 AddHelpWidget(lwoh, hbox);
+      }
+   }
+
+   AddHelpParagraph(lwoh, _("<b>Encoding algorithm</b>\n\n"
+     "This option affects the speed of generating RS03 error correction data.\n"
+     "dvdisaster can either use a generic encoding algorithm using 32bit or 64bit "
+     "wide operations running on the integer unit of the processor, or use "
+     "processor specific extensions.\n\n"
+     "Available extensions are SSE2 for x86 based processors and AltiVec "
+     "on PowerPC processors. These extensions encode with 128bit wide operations "
+     "and will usually provide the fastest encoding variant. If \"auto\" is selected, the "
+     "SSE2/AltiVec algorithms will be selected if the processor supports them; "
+     "otherwise the 64bit algorithm will be used."
+			    ));
 }
